SystemOrganization addCategory: #'Pier-Jetsam-Environment'!

!PRStructure class methodsFor: '*pier-jetsam-environment' stamp: 'kph 4/23/2008 04:37'!
descriptionEnvironmentJetsam: desc
	^ PRStructureDescription new
		accessor: #localEnvironment;
		addCondition: [ :v | v respondsTo: #document ]; "hack because #hasDocument is inconsistent"
		addCondition: 
				[ :value | 
				value contents isEmpty or: [ 
				value embeddedComponents anySatisfy: [ :each | each componentClass includesBehavior: PRContentsWidget ]] ]
			labelled: 'Invalid environment, make sure to embed the contents widget';
		group: 'Layout Settings';
		label: 'Environment';
		priority: 800;
		beSetting;
		yourself! !

!PRStructure class methodsFor: '*pier-jetsam-environment' stamp: 'kph 4/22/2008 00:50'!
descriptionStyleSheetRemove: env
	^ nil! !

!PRStructure methodsFor: '*pier-jetsam-environment' stamp: 'kph 4/22/2008 00:35'!
childWithName: aName

	self enumerator 
		do: [ :each | each name = aName ifTrue: [ ^ each ] ].
	^ nil
	

"
(PRKernel instanceNamed: 'Pier') root childWithName: 'environment'
"! !

!PRStructure methodsFor: '*pier-jetsam-environment' stamp: 'kph 4/22/2008 06:13'!
environmentLookup: path
	"Answer the (inherited) environment structure of the receiver, if none is available a default one is created."

	^ (self lookupPath: path in: self localEnvironment) ifNil: [
		self isRoot
			ifTrue: [ nil ]
			ifFalse: [ self parent environmentLookup: path ] ]! !

!PRStructure methodsFor: '*pier-jetsam-environment' stamp: 'kph 4/22/2008 06:34'!
lookupPath:  aPath in: structure

	structure ifNil: [ ^ nil ].

	^ PRPathLookup start: structure path: aPath onError: [ nil ]


"
(PRKernel instanceNamed: 'Pier') root pathLookup: 'environment/footer'
"! !

!PRStructure methodsFor: '*pier-jetsam-environment' stamp: 'kph 4/23/2008 05:22'!
updateChild: aStructure
	"Add ==aStructure== as child to the receiver."
 
	(self childWithName: aStructure name) ifNotNilDo: [ :child | child remove ].
	
	^ self addChild: aStructure! !

!PRPierFrame methodsFor: '*pier-jetsam-environment' stamp: 'kph 4/22/2008 07:58'!
forbidden: aRequest
 
	^ nil
 ! !

PRPierFrame subclass: #PRPierFrameCentral
	instanceVariableNames: 'environmentCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Jetsam-Environment'!

!PRPierFrameCentral commentStamp: 'kph 4/23/2008 05:05' prior: 0!
This Seaside Root component becomes the bottleneck for creating default content, and for determining policies, such as where to find layouts and sylesheets.

PRPierFrameCentral implements a refactoring of standard Pier behaviour.

BUT: We do not have to have an editable environment if we dont want one AND If we add one it gets initialized for us.
To add an environment, add an empty page wherever you like, name it whatever you like, and set a localEnvironment to it.
 
This supplies and caches a default environment, until the user supplies one.
The cached default environment is not editable from the web interface.

1. Style Libraries (currently manually added in seaside/config) 
2. Script Libraries (currently manually added in seaside/config) 
3. Policy for finding Layout from the structure. (currently in PRStructure) 
4. Policy for finding StyleSheets (currently in PRStructure)
5. Default Environment (currently in PRStructure)
6. Default Layout (currently in PRStructure)
7. Default Kernel Content (currently in PRKernel class)
8. Not Found Handler (currently in PRMain)
9. Forbidden Handler (currently in PRMain)
10. XHTML Doctype/Strict etc. (already in PRFrame)
11. Ability to override the body cssClass (currently obtained from the structure)
12. Option to adapt the #settingsDescription for components being editted. (can add/remove fields)


!

PRPierFrameCentral subclass: #PRPierBasicBP
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Jetsam-Environment'!

!PRPierBasicBP commentStamp: 'kph 4/23/2008 03:15' prior: 0!
This provides a pluggable method to provide alternative organization policies.

PRPierBasic implements the following policies.

We move the environment template, from /environment to
/environment/layout
/environment/style

If we are looking for a part, say 'layout' and we dont find one at all, we initialize that part in the nearest environment, not the root environment.

If the root's environment is not set, then it will use the default environment location. AND, it will use the environment as defined here, ignoring attempts to edit it via the web interface. To open the environment for editing, change the root environment to /env!

PRPierBasicBP subclass: #PRPierAdmin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Jetsam-Environment'!

!PRPierAdmin commentStamp: 'kph 4/22/2008 05:31' prior: 0!
This frame displays the site using the layout in environment/admin.

This allows a second seaside application to be set up which shows an admin layout of the site. This application can be configured to need authorization before viewing.

Thus, two (or potentially more) levels of security can be configured without using Pier-Security.!

!PRPierAdmin methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 06:08'!
environment
 
	^ self environmentCache ifNil: [ self environmentCacheInitialize ]! !

!PRPierAdmin methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 06:01'!
environmentDefault: page

	page childrenDecoration remove.	 

	^ page
		 ! !

!PRPierAdmin methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 06:00'!
layout
 
	^  (self environment childWithName: self layoutName)
			ifNil: [ self layoutInitialize ]! !

!PRPierAdmin methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 05:34'!
layoutContents

^  '<div class="container">
	<div class="header column span-24">
		<div class="title column span-20">+title+</div>
		<div class="search column span-4 last">+search+</div>
		<div class="children column span-24 last">+children+</div>
	</div>
	<div class="body column span-24">
		<div class="sidebar column span-4">+sidebar+</div>
		<div class="contents column span-20 last">+contents+</div>
	</div>
	<div class="footer column span-24">+footer+</div>
</div>'! !

!PRPierAdmin methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 05:42'!
layoutDefault: env

	super layoutDefault: (env updateChild: ((PRPage named: self layoutName)
			contents: self layoutContents;
			yourself)).
			
			
 
	! !

!PRPierAdmin methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 06:04'!
layoutName

	^ 'admin'! !

!PRPierAdmin methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 04:54'!
layoutPageName

	^ 'admin'! !

!PRPierBasicBP class methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 07:12'!
defaultKernelRoot

	^ PRKernel defaultRoot.! !

!PRPierBasicBP methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 05:27'!
environmentDefault: page

	page childrenDecoration remove.	 
	 
	page
		contents: 
'-*layout*
-*style*'.
 
^ page! !

!PRPierBasicBP methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 05:18'!
layout
 
	^ ((self context structure environmentLookup: self layoutName)   
			ifNil: [ self environment childWithName: self layoutName  ])  
				ifNil: [ self layoutInitialize ]! !

!PRPierBasicBP methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 05:18'!
layoutContents

^  '<div class="container">
	<div class="header column span-24">
		<div class="title column span-20">+../title+</div>
		<div class="search column span-4 last">+../search+</div>
		<div class="children column span-24 last">+../children+</div>
	</div>
	<div class="body column span-24">
		<div class="sidebar column span-4">+../sidebar+</div>
		<div class="contents column span-20 last">+../contents+</div>
	</div>
	<div class="footer column span-24">+../footer+</div>
</div>'! !

!PRPierBasicBP methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 05:23'!
layoutDefault: env

 
	env updateChild: ((PRPage named: self layoutName)
			contents: self layoutContents;
			yourself).
	env updateChild: ((PRPage named: 'title')
			contents: '!!*/*';
			yourself).
	env updateChild: ((PRComponent named: 'search')
			componentClass: PRAjaxSearchWidget;
			yourself).
			
	env updateChild: ((PRComponent named: 'children')
			componentClass: PRChildrenWidget;
			write: 1 using: PRChildrenWidget descriptionLevel;
			write: true using: PRChildrenWidget descriptionHighlightPath;
			yourself).
			
	env 	updateChild: ((PRPage named: 'sidebar')
			updateChild: ((PRComponent named: 'navigation')
				componentClass: PRChildrenWidget;
				write: 2 using: PRChildrenWidget descriptionLevel;
				write: true using: PRChildrenWidget descriptionExpand;
				yourself);
			updateChild: ((PRComponent named: 'views')
				componentClass: PRViewsWidget;
				yourself);
			updateChild: ((PRComponent named: 'commands')
				componentClass: PRCommandsWidget;
				yourself);
			contents: '+navigation+
+views+
+commands+';
			yourself).
			
	env 	updateChild: ((PRComponent named: 'contents')
			componentClass: PRContentsWidget;
			write: '%c' using: PRContentsWidget descriptionHeading;
			yourself).
	env updateChild: ((PRPage named: 'footer')
			contents: 'Powered by *Squeak>http://www.squeak.org*, *Seaside>http://www.seaside.st*, *Magritte>http://www.lukas-renggli.ch/smalltalk/magritte*
		and *Pier>http://www.lukas-renggli.ch/smalltalk/pier*.';
			yourself).
	 ! !

!PRPierBasicBP methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 05:07'!
layoutInitialize
 
	self layoutDefault: self environment.

	^ self layout! !

!PRPierBasicBP methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 03:14'!
layoutName

	^ 'layout'
! !

!PRPierBasicBP methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 06:10'!
styleSheets

   ^ Array with: ((self structure environmentLookup: 'style') ifNil: [ ^ #() ])! !

!PRPierBasicBP methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 02:06'!
updateEnvironmentContents

	"this is a hook called on every render,
	that you can use to update the environment when editting the templates."
	
	self environment contents: self environmentDefault contents.! !

PRPierBasicBP subclass: #PRPierBasicCWS
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Jetsam-Environment'!

!PRPierBasicCWS methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 00:19'!
cssLayoutChoice

	^ 
	
	" #layoutnavtoplocalleftCss "
	" #layoutnavtopsubrightCss "
	" #layoutnavtop3colCss "
	" #layoutnavtop1colCss "
	" #layoutnavleft1colCss "
	" #layoutnavleft2colCss 
	" #layoutnavtoplocalleftCss ! !

!PRPierBasicCWS methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 01:53'!
layoutContents

^  '<div id="page"> 
            <div id="header" class="clearfix">
			+../header+
                <hr />
            </div> 
            <div id="content" class="clearfix">
                <div id="main">
				+../contents+
                    <hr />
                </div>
                
                <div id="sub">
				+../sub+
                    <hr />
                </div>
                
                <div id="local">
				+../sidebar+ 
                   <hr />
                </div>
                
                <div id="nav">
				<div class="wrapper">
                	<h2 class="accessibility">Navigation</h2>
    				</div>
                <ul class="clearfix">
				+../children+
                    <hr />
                </div>
                
            </div> 
            
            <div id="footer" class="clearfix">
				+../footer+
            </div> 
            
        </div> 
        
        <div id="extra1">&nbsp;</div>
        <div id="extra2">&nbsp;</div>'! !

!PRPierBasicCWS methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 00:14'!
updateRootLibraries: aHtmlRoot
	
	"we can include libraries here, so the user does not have to add them manually to the config"
	
	PRContentWithStyleLibrary default in: [ :lib  |  
	
		aHtmlRoot stylesheet addScreen; url: (lib urlOf: #toolsCss).
		
		aHtmlRoot stylesheet addScreen; url: (lib urlOf: #typoCss).
		
		aHtmlRoot stylesheet addScreen; url: (lib urlOf: #formsCss).
	 
		aHtmlRoot stylesheet addScreen; url: (lib urlOf: self cssLayoutChoice).
			
		aHtmlRoot stylesheet addScreen; url: (lib urlOf: #layoutCss).

	]! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 07:24'!
adaptSettingsDesctiption: desc

	"here you get a chance to override the settingsDescriptions of PRSettingsCommand"

	"not wired up yet"
	^ desc! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 03:28'!
buildChildren
 	  
	^ Set new
		addAll: (self componentsOf: self layout);
		addAll: (self componentsOf: self structure);
		yourself! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 07:23'!
cssClass

^ self structure cssClass! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 04:15'!
environment

	^ (((self context structure environmentLookup: '.') 
			ifNotNilDo: [ :page | page contents ifEmpty: [ self environmentDefault: page ]. page ])
			ifNil: [ self environmentCache ]) ifNil: [ self environmentCacheInitialize ]! !

!PRPierFrameCentral methodsFor: 'accessing' stamp: 'kph 4/23/2008 03:58'!
environmentCache
	"Answer the value of environmentCache"

	^ environmentCache! !

!PRPierFrameCentral methodsFor: 'accessing' stamp: 'kph 4/23/2008 03:58'!
environmentCache: anObject
	"Set the value of environmentCache"

	environmentCache := anObject! !

!PRPierFrameCentral methodsFor: 'accessing' stamp: 'kph 4/23/2008 04:01'!
environmentCacheInitialize

	environmentCache := self environmentDefault: (PRPage named: 'env').
	environmentCache outgoingReferences do: [ :each | each refresh ].
	^ environmentCache! !

!PRPierFrameCentral methodsFor: 'accessing' stamp: 'kph 4/23/2008 04:07'!
environmentDefault: page

	page childrenDecoration remove.	 

	^ page
		addChild: ((PRPage named: 'title')
			contents: '!!*/*';
			yourself);
		addChild: ((PRComponent named: 'search')
			componentClass: PRAjaxSearchWidget;
			yourself);
		addChild: ((PRComponent named: 'children')
			componentClass: PRChildrenWidget;
			write: 1 using: PRChildrenWidget descriptionLevel;
			write: true using: PRChildrenWidget descriptionHighlightPath;
			yourself);
		addChild: ((PRPage named: 'sidebar')
			addChild: ((PRComponent named: 'navigation')
				componentClass: PRChildrenWidget;
				write: 2 using: PRChildrenWidget descriptionLevel;
				write: true using: PRChildrenWidget descriptionExpand;
				yourself);
			addChild: ((PRComponent named: 'views')
				componentClass: PRViewsWidget;
				yourself);
			addChild: ((PRComponent named: 'commands')
				componentClass: PRCommandsWidget;
				yourself);
			contents: '+navigation+
+views+
+commands+';
			yourself);
		addChild: ((PRComponent named: 'contents')
			componentClass: PRContentsWidget;
			write: '%c' using: PRContentsWidget descriptionHeading;
			yourself);
		addChild: ((PRPage named: 'footer')
			contents: 'Powered by *Squeak>http://www.squeak.org*, *Seaside>http://www.seaside.st*, *Magritte>http://www.lukas-renggli.ch/smalltalk/magritte*
		and *Pier>http://www.lukas-renggli.ch/smalltalk/pier*.';
			yourself);
		contents: '<div class="container">
	<div class="header column span-24">
		<div class="title column span-20">+title+</div>
		<div class="search column span-4 last">+search+</div>
		<div class="children column span-24 last">+children+</div>
	</div>
	<div class="body column span-24">
		<div class="sidebar column span-4">+sidebar+</div>
		<div class="contents column span-20 last">+contents+</div>
	</div>
	<div class="footer column span-24">+footer+</div>
</div>';
	yourself! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 07:58'!
forbidden: aRequest
 
	| command page |

	self context: (PRContext kernel: self kernel).

	page := (self kernel root lookupPath: 'forbidden' in: self kernel root) ifNil: [  ^ nil  ].
	
	PRCurrentContext use: self during: [
		command :=	 page defaultCommandClass  new
			setContext: self context;
			initialRequest: aRequest;
			yourself.
			
	self context
		setStructure: page;
		setCommand: command.
	self context isValid
			ifFalse: [ ^ nil ] ].
		
	^ self
! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 08:06'!
forbiddenRequest: aRequest

 	(self forbidden: aRequest) ifNil: [  ^ self session returnResponse: (WAResponse forbidden: aRequest url)  ].
	
	self visiblePresentersDo: [ :each | each initialRequest: aRequest ].
	
	(self session mainClass new renderLoopClass new root: self) run! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 03:50'!
layout

	^ self environment! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 07:25'!
renderContentOn: html
	self context renderer
		start: self layout
		in: self on: html! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/23/2008 03:29'!
root

^ self context kernel root! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 07:31'!
styleSheets
	"Answer the a collection of stylesheets of the receiver."
	
	^ (self structure parents
		collect: [ :each | each localStyleSheet ])
		reject: [ :each | each isNil ]! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 08:43'!
updateRoot: aHtmlRoot
	self script ifNotNilDo: [:script | aHtmlRoot addScript: script].
	self style ifNotNilDo: [:style | aHtmlRoot addStyle: style].

	self updateRootLibraries: aHtmlRoot.
	
	"NOTE XML PREAMBLE REMOVED FOR IE6 - kph"
	aHtmlRoot docType: '<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'.
	aHtmlRoot htmlAttributes at: 'xmlns' put: 'http://www.w3.org/1999/xhtml'.
	aHtmlRoot bodyAttributes addClass: self cssClass.

	self styleSheets
		do: [ :each | aHtmlRoot stylesheet url: (each file urlOn: aHtmlRoot) ].

	self structure isRoot
		ifFalse: [ aHtmlRoot link beUp; url: (self context structure: self structure parent) url ].

	aHtmlRoot link beTop; url: (self context structure: self structure root) url.

	aHtmlRoot meta name: 'generator'; content: 'Pier' 
  ! !

!PRPierFrameCentral methodsFor: 'as yet unclassified' stamp: 'kph 4/22/2008 08:45'!
updateRootLibraries: aHtmlRoot
	
	"we can include libraries here, so the user does not have to add them manually to the config"
	
	PRBlueprintLibrary default updateRoot: aHtmlRoot! !

!PRKernel class methodsFor: '*pier-jetsam-environment-override' stamp: 'kph 4/22/2008 07:13'!
named: aString
	^ self named: aString root: nil! !

!PRPierMain methodsFor: '*pier-jetsam-environment-override' stamp: 'kph 4/22/2008 07:57'!
forbidden: aRequest

	| root |
	
	Preferences haltOnForbidden  ifTrue: [ self halt ].

	"self return: (WAResponse forbidden: aRequest url)"

	root := self createRoot forbidden: aRequest.
	
	root ifNil: [ ^ self return: (WAResponse forbidden: aRequest url) ].
	
	root visiblePresentersDo: [ :each | each initialRequest: aRequest ].

	(self renderLoopClass new root: root) run! !

!PRPierMain methodsFor: '*pier-jetsam-environment' stamp: 'kph 4/22/2008 07:11'!
kernelRoot

^ self kernel root ifNil: [ self kernel root: self rootClass defaultKernelRoot ]! !

!PRPierMain methodsFor: '*pier-jetsam-environment-override' stamp: 'kph 4/22/2008 07:19'!
start: aRequest
	self kernelRoot.
	self context: (PRContext kernel: self kernel).
	PRCurrentContext use: self during: [
		self context
			setStructure: (self structureFrom: aRequest);
			setCommand: (self commandFrom: aRequest).
		self context isValid
			ifFalse: [ self forbidden: aRequest ] ].
	super start: aRequest! !
